;------------------------------------------------------------------------------
;
; Uhrenprogramm f. 5MHz und 8MHz
;------------------------------------------------------------------------------
; EMR Uhr mit UB8820M
; radio fersehen elektronik 9/89
; Seite 590 bis Steite 594
;------------------------------------------------------------------------------
; entwickelt von:
; Gerd Schachtschneider 
; Max Planck Institute
; IT-Department
; Sandtorstr. 1 
; 39106 Magdeburg
; Germany
;------------------------------------------------------------------------------
; Registerprogrammierung entsprechend der Quarzfrequenz
;------------------------------------------------------------------------------
; im Programm welches Register mit Wert bei   5MHz     8MHz   laden
;    INI               R4                      7D       64
;    INI               R5                      17       2B
;    INI               R3                      1B       2B
;    ALA               R11                     02       03
;    ALA               R11                     02       03
;    TON               PRE1                    13       1B
;    WE1               PRE1                    0F       17
;    WE2               PRE1                    0F       13
;    KZW               PRE1                    0F       13
;------------------------------------------------------------------------------
     GLOBALS ON
;
;  TRUE/FALSE equates 
;
TRUE  .EQU 1
FALSE .EQU 0

;- default 5MHz
; FLASE = 5MHz
; TRUE = 8MHz
EMR8MHZ .EQU  FALSE


 IF EMR8MHZ

INI_R4     .EQU   %64
INI_R5     .EQU   %2B
INI_R3     .EQU   %2B
ALA_R11    .EQU   %03
TON_PRE1   .EQU   %1B
WE1_PRE1   .EQU   %17
WE2_PRE1   .EQU   %13
KZW_PRE1   .EQU   %13

 ELSE ; 5MHz

INI_R4     .EQU   %7D
INI_R5     .EQU   %17
INI_R3     .EQU   %1B
ALA_R11    .EQU   %02
TON_PRE1   .EQU   %13
WE1_PRE1   .EQU   %0F
WE2_PRE1   .EQU   %0F
KZW_PRE1   .EQU   %0F

 ENDIF

;------------------------------------------------------------------------------
; Registerprogrammierung entsprechend der Quarzfrequenz
;------------------------------------------------------------------------------
;IDV_WECK1_h        .EQU   5 ; ( h   )     1. Weckzeit
;IDV_WECK1_min      .EQU  30 ; ( min )     1. Weckzeit
;IDV_WE1_PAU_WIED   .EQU   6
;IDV_WE2_PAU_WIED   .EQU  30
;
;
;IDV:    DB 5,30    ;  ( h : min )     1. Weckzeit
;        DB 6,30    ;  ( h : min )     2. Weckzeit
;		    DB 3,5     ;  ( s : min )  Pausenzeit + Weckerwiederholung WE1
;		    DB 3,5     ;  ( s : min )  Pausenzeit + Weckerwiederholung WE2
;		    DB 12,19   ;  ( h : h   ) Beginn - und Endzeit des Stundentones
;		    DB 0,0     ;  ( h : min ) 1. Einschaltzeit
;		    DB 0,0     ;  ( d : m   ) mit Datum
;		    DB 0,0     ;  ( h : min ) 1. Ausschaltzeit
;		    DB 0,0     ;  ( d _ m   ) mit Datum
;		    DB 30,5,5  ;  ( s : s : s ) Anzeigenwechsel im PR0 ( Uhr;Temp;Datum)
;		    DB 0,0     ;  ( C : C )   Temperaturregler mit Hysterese
;		    DB 0,0     ;  ( h : min ) 2. Einschaltzeit
;		    DB 0,0     ;  ( d : m   ) mit Datum
;		    DB 0,0     ;  ( h : min ) 2. Ausschaltzeit
;		    DB 0,0     ;  ( d : m   ) mit Datum
;		    DB 0,0     ;  ( min : s ) Kurzzeitwecker
;------------------------------------------------------------------------------
;		    
;------------------------------------------------------------------------------
; Interrupt Vectors
;------------------------------------------------------------------------------

        .org    %0           ; Setup interrupt vectors
 
INT:    .word   INI          ; Interrupt 0  IRQ0 ( P32 )
        .word   INI          ;     "     1  IRQ1 ( P33 )
        .word   INI          ;     "     2  IRQ2 ( P31 )
        .word   INI          ;     "     3  IRQ3 ( P30 / RXD )
IVA:    .word   IVR          ; Interruptvektoradresse %07F2 H  IRQ4 ( T0 / TXD )
        .word   INI          ;     "     5  IRQ5 ( T1 )
;------------------------------------------------------------------------------
;   Initialisierung
;------------------------------------------------------------------------------
INI:    SRP     #%F0         ; Registergruppe % 0F0
        LD      R8,#%45      ; Maske P01M
        CLR     R6           ; MaskeP2M
        LD      R7,#%01      ; Maske P3M
        LD      R15,#%80     ; Stackpointer auf % 0080 H laden
        DI                   ; Interrupt erstmalig sperren
        LD      R11,#%10     ; IMR laden  ( T0 zugelassen )
        LD      R9,#%0F      ; IPR laden  ( T0 > T1 )
        LD      R4,INI_R4    ; T0 laden   ( : 125 ) bei 5MHz
        LD      R5,INI_R5    ; PRE0 laden ( :   5 ) / 600 * 8
        LD      R3,INI_R3    ; PRE1 laden ( :   6 )
        CLR     P1           ; Port 1 lschen
        CLR     P2           ; Port 2 lschen
        LD      P3,#%10      ; Relais Port 3 inaktiv
        LD      R1,#%BF      ; TMR laden ( T0;T1 aktivieren / P36 = T1 )
        
        SRP     #%40
        LD      R0,#%04      ; Lschen der Register ab % 04
        LD      R1,#%3C      ;                      bis % 3F
IN1:    CLR     @R0          ; idiziert lschen
        INC     R0           ; 04 - 40
        DJNZ    R1,IN1       ; 60 * ?
        
        SRP     #%10
        LD      R14,#%06     ; Individuelle Daten ab Speicherplatz
        LD      R15,#%E0     ; % 06E0 H
        LD      R11,#%40     ; zu laden ab Reg. % 40
        LD      R10,#%20     ;         bis Reg. % 5F
IN2:    LDCI    @R11,@RR14   ; Daten holen
        DJNZ    R10,IN2      ; 32 * ?
        
        EI                   ; Interrupt zulassen
        LD      R2,#%80      ; kein Stellmodus
        LD      R7,#%0C      ; Programmspeicher auf PR0
        LD      R12,#%06     ; Segmentdaten ab % 06D0 H
        LD      R15,#%0C     ; Tastaturabfragereg. auf PR0
        CALL    ALA
;------------------------------------------------------------------------------
; Hauptprogramm
;------------------------------------------------------------------------------
HPR:    DI                   ; Interrupt f. Tastaturabfrage sperren
        CALL    TSA          ; --> Tastaturabfrage
        EI                   ; Interrupt wieder zulassen
        CALL    PLD          ; --> Programm lt. Tastatur laden
        CALL    RKO          ; --> Registerkorrektur
        CALL    WE1          ; --> Wecker 1
        CALL    WE2          ; --> Wecker 2
        CALL    RGL          ; --> Temperaturregler
        CALL    TON          ; --> Stundenton
        CALL    REL          ; --> Relaisein -und ausschaltzeiten
        CALL    DAT          ; --> Datum
        CALL    KZW          ; --> Kurzzeitwecker
        CALL    SPU          ; --> Stopuhr
        JR      HPR          ; --> zurck zum Anfang !
;------------------------------------------------------------------------------
; Abfrage d. Zusatztaste im Interruptzyklus
;------------------------------------------------------------------------------        
TAS:    CLR     RP
        TM      P0,#%80      ; P 07 = L ?
        JR      Z, TA1       ; --> ja = Z-Ta gedrckt
        CLR     R12          ; nein, Zhlreg. auf 00F0 voreinstellen
        LD      R13,#%F0     ; 1 Bit = 1ms
TA1:    INCW    RR12         ; Xa = Xi + 1ms
        RET
;------------------------------------------------------------------------------
; Timer fr Uhr im Interruptzyklus
;------------------------------------------------------------------------------
TIM:    SRP     #%20
                             ; Durchlauf erfolgt alle 1ms
        ADD     R0,#%10      ; Reg. f. 1/1000 s
        DA      R0           ; Xa = Xi + 10  ( 10 * 1ms )
        JR      NC,TI6       ; Xa = 100 --> nein
                             ; Durchlauf alle 10ms
        TCM     R12,#%80     ; Abtastreg. f. RPM;PRP ber 1,27s ?
        JR      Z,TI5        ; --> ja = Wert halten
        INC     R12          ; Xa = Xi * 10ms
TI5:    ADD     R15,#%01     ; Abtastreg. f. Tonlnge WE1;WE2;TON;KZW
        DA      R15          ; Xa = Xi * 10 ms
        ADD     R1,#%01      ; Reg. f. 1/100 s
        DA      R1           ; Xa = Xi + 1  ( 100 * 10ms )
        JR      NC,TI6       ; Xa = 100 --> nein
                             ; Durchlauf alle 1s
        ADD     R2,#%01      ; Reg. f. 1s
        DA      R2           ; Xa = Xi + 1
        SUB     R14,#%01     ; Abtastreg. f. Wechselzeit PR0; Temp.; Datum
        DA      R14          ; Xa = Xi + 1s
        JR      NC,TI1       ; Xa = 0 --> nein
        CLR     R14          ; ja, Abtastreg. auf null halten
TI1:    SUB     R13,#%01     ; Abtastreg. f. Pausenzeit WE1;WE2
        DA      R13          ; Xa = Xi + 1s
        JR      NC,TI2       ; Xa = 0 --> nein
        CLR     R13          ; ja, Abtastreg. auf null halten
TI2:    TM      %3F,#%01     ; Kurzzeitwecker aktiv ? ( Bit 0 = 1 )
        JR      Z,TI6        ; --> nein
        SUB     %5F,#%01     ; ja, Zhlreg.(min; s ) decrementieren
        DA      %5F          ; Xa = Xi - 1s
        JR      NC,TI6       ; Xa < 0 --> nein
        LD      %5F,#%59     ; ja, Sekunden auf 59s stellen und
        SBC     %5E,#%00     ;     von Minuten 1 subtrahieren
        DA      %5E
                             ; Durchlauf erfolgt alle 10ms
TI6:    CP      %12,#%26     ; Uhrzeit (h) im Stellmodus ?
        JR      NZ,TI3       ; --> nein
        CLR     R2           ; ja, Reg. f. Sekunden lschen und
        LD      %16,#%0E     ; Doppelpunkt ST3 stndig aktiv
TI3:    CP      R2,#%60      ; Reg. f. (s) >= 60 ?
        JR      C,TI4        ; --> nein
                             ; Durchlauf erlogt alle 1 min.
        CLR     R2           ; ja, Reg. f. (s) lschen und
        ADD     R6,#%01      ;     Reg. f. (min) 1 addieren
        DA      R6           ; Xa = Xi + 1
        CP      R6,#%60      ; Reg. f. (min) >= 60 ?
        JR      C,TI4        ; --> nein
                             ; Durchlauf erfolgt alle 1h
        CLR     R6           ; ja, Reg. f. (min) lschen und
        ADD     R5,#%01      ;     Reg. f. (h) 1 addieren
        DA      R5           ; Xa = Xi + 1
        CP      R5,#%24      ; Reg. f. (h) >= 24 ?
        JR      C,TI4        ; --> nein
                             ; Durchlauf erfolgt alle 1d
        CLR     R5           ; ja, Reg. f. (h) lschen und
        ADD     R7,#%01      ;     Reg. f. (d) 1 addieren
        DA      R7           ; Xa = Xi + 1
TI4:    RET 
;------------------------------------------------------------------------------
; Anzeigeroutine im Interruptzyklus
;------------------------------------------------------------------------------
ANZ:    SRP     #%10
        CLR     P1           ; Digitatellen + LED's inaktiv
        SRA     R3           ; Reg. f. Stellendurchlauf laden
                             ; Anzeigestelle 1 -
ST1:    CP      R3,#%10      ; Reg. f. Stelle = St1 ?
        JR      NZ,ST2       ; --> nein
        LD      R13,R4       ; ja, Daten f. ST1 vom Zsp. laden ( St1,St2)
        SWAP    R13          ;     (ST2,ST1)
        LD      R1,%35       ;      Komma f. St1 vom Zsp. laden ( #%04 )
        JR      SCA          ; --> Segmentcodeausgabe
                             ; Anzeigestelle 2 -
ST2:    CP      R3,#%08      ; Reg. f. Stelle = St2 ?
        JR      NZ,ST3       ; --> nein
        LD      R13,R4       ; ja, Daten f. St2 vom Zsp. laden ( St1, St2 )
        LD      R1,#%36      ;     Komma f. St2 vom Zsp. laden ( #% 04 )
        CP      R6,#%FD      ; Daten f. St3 = Datumanzeige ?
        JR      NZ,AN1       ; --> nein
        ADD     R1,#%04      ; ja, Komma fr Datum addieren
        AND     R1,#%04      ;     im Stellmodus ? ja, Komma aus !
AN1:    JR      SCA          ; --> Segmentcodeausgabe
                             ; Anzeigestelle 3 -
ST3:    CP      R3,#%04      ; Reg. f. Stelle = St3 ?
        JR      NZ,ST4       ; --> nein
        CLR     R1           ; ja, AR f. Blinkcode lchen
        CP      %21,#%50     ; Reg. f. 1/100 s < 50 ? ( = 0,5s )
        JR      C,AN2        ; --> ja
        LD      R13,R6       ; nein, Blinkcode f. St3 holen
        SWAP    R13          ;       Blinkcode im NWT
        AND     R13,#%0F     ;       nur Blinkcode
        ADD     R13,#%D0     ;       Segmentcode ab Adresse % 0600 H
        LDC     R1,@RR12     ;       AR f. Blinkcode <-- Adresse von RR12
AN2:    LD      R13,R6       ; Daten f. St3 vom Zsp. laden ( blinkc. St3, St3 )
        JR      SCA          ; --> Segmentcodeausgabe
                             ; Anzeigestelle 4 -
ST4:    CP      R3,#%02      ; Reg. f. Stelle = St4 ?
        JR      NZ,ST5       ; --> nein
        LD      R13,R5       ; ja, Daten f. St4 vom Zsp. laden ( St4,St5 )
        SWAP    R13          ;     ( St5,St4 )
        LD      R1, %37      ;     Komma f. St4 vom Zsp. laden ( #% 04 )
        JR      SCA          ; --> Segmentcodeausgabe
                             ; Anzeigestelle 5 -
ST5:    LD      R13,R5       ; Daten f. St5 vom Zsp. laden ( St4, St5 )
        LD      R1,%38       ; Komma f. St5 vom Zsp. laden ( #% 04 )
        CP      R6,#%FD      ; Daten f. St5 = Datumanzeige ?
        JR      NZ,AN3       ; --> nein
        ADD     R1,#%04      ; ja, Komma addieren
        AND     R1,#%04      ;     im Stellmodus ? ja, Komma aus !
AN3:    CALL    SCA          ; Segmentcodeausgabe
        LD      R3,#%20      ; Reg, f. Stellendurchlauf auf St1 voreinstellen
        RET
;------------------------------------------------------------------------------
; Segmentcodeausgabe
;------------------------------------------------------------------------------        
SCA:    AND     R13,#%0F     ; nur der NWT gilt
        ADD     R13,#%D0     ; ab Adrese % 06D0 H
        LDC     R0,@RR12     ; AR <-- Segmentcode von Speicherplatz holen
        OR      R0,R1        ; Segmentcode + Blinkcode; Komma
        LD      P2,R0        ; --> zum Port 2 laden
;-----------------------------
; Digit + LED-Ausgabe
;-----------------------------
LED:    LD      R0,R9        ; AR <-- Reg. f. LED-Daten ( normal, blinken )
        CP      %21,#%50     ; Reg. f. 1/100s < 50 ? ( = 0,5 s )
        JR      C,LE1        ; --> ja = ohne Blink-LED
        SWAP    R0           ; ( normal,blinken) --> (blinken,normal)-LED
        AND     R0,#%E0      ; nur Blink-LED
        OR      R0,R9        ; Blink + normal -LED
LE1:    AND     R0,#%E0      ; nur P15;P16;P17
        OR      R0,R3        ; LED-Daten + Digitstelle
        LD      P1,R0        ; LED-Daten;Digitstelle --> zum Port 1
        RET
;------------------------------------------------------------------------------
; Alarmschleife nach Netzzuschaltung
;------------------------------------------------------------------------------       
ALA:    INC     R8           ; erstmals von 00 bis 64 ( =100 bis 1KHz )
        LD      T1,R8        ; dann von 32 bis 64     ( =  2 bis 1KHz )
        LD      R11,ALA_R11  ; 2 * 256 * ca. 0,0016ms ( = 0,8 ms/ 1KHz )
AL1:    TM      P0,#%80      ; Zusatztaste gedrckt ?
        JR      Z,AL4        ; --> ja ( Alarm aus )
        CP      %26,#%01     ; Alarmton 1 min aktiv ?
        JR      Z,AL4        ; --> ja ( Alarm aus )
        DJNZ    R10,AL1      ; 256 *
        DJNZ    R11,AL1      ; 2 * 256
        CP      R8,#%64      ; 1KHz erreicht ?
        JR      NZ,ALA       ; --> nein ( Frequenz verringern )
AL2:    DEC     R8           ; ja, Alarmton erhhen
        LD      T1,R8        ; zum Port 36
        LD      R11,ALA_R11  ; 2*256*ca.0,0016ms ( Schwellton alle 0,8 s )
AL3:    TM      P0,#%80      ; Zusatztaste gedrckt ?
        JR      Z,AL4        ; --> ja ( Alarm aus )
        DJNZ    R10,AL3      ; 256 *
        DJNZ    R11,AL3      ; 2 * 256
        CP      R8,#%32      ; 2 KHz erreicht ?
        JR      ALA          ; wiederholen !
AL4:    CALL    TMA          ; --> Alarmton aus !
        RET
;------------------------------------------------------------------------------
; Tonabschaltung
;------------------------------------------------------------------------------
TMA:    AND     TMR,#%33     ; T1;PRE1 im TMR nicht neu laden !
        AND     P3,#%BF      ; Port 36 auf Low
        RET
;------------------------------------------------------------------------------
; Abfrage Matrixtatatur
;------------------------------------------------------------------------------       
TSA:    SRP     #%10
TZ1:    TM      P3,#%01      ; Zeile 1(P30) = L ? ( PRP; PR0; PRM )
        JR      NZ,TZ2       ; --> nein
        CLR     R15          ; Programmverteiler % 0700; 070C; 0718
        JR      TS1          ; --> Spaltenabfrage
TZ2:    TM      P3,#%02      ; Zeile 2(P32) = L ? ( PR1; PR2; PR3 )
        JR      NZ,TZ3       ; --> nein
        LD      R15,#%03     ; Programmverteiler % 0703; 070F; 071B
        JR      TS1          ; --> Spaltenabfrage
TZ3:    TM      P3,#%04      ; Zeile 3(P32) = L ? ( PR4; PR5; PR6 )
        JR      NZ,TZ4       ; --> nein
        LD      R15,#%06     ; Programmverteiler % 0706; 0712; 071E
        JR      TS1          ; --> Spaltenabfrage
TZ4:    TM      P3,#%08      ; Zeile 4(P33) = L ? ( PR7; PR8; PR9 )
        JR      NZ,SM1       ; --> nein
        LD      R15,#%09     ; Programmverteiler % 0709; 0715; 0721
TS1:    TCM     P1,#%01      ; Spalte 1(P10) = H ? (PRP; PR1; PR4; PR7)
        JR      NZ,TS2       ; --> nein
                             ; Programmverteiler % 0700; 0703; 0706; 0709 <fehlt hier was ?>
        JR      STM          ; --> Abfrage Stellmodus
TS2:    TCM     P1,#%04      ; Spalte 2(P12) = H ? (PR0; PR2; PR5; PR8)
        JR      NZ,TS3       ; --> nein
        ADD     R15,#%0C     ; Programmverteiler % 070C; 070F; 0712; 0715
        JR      STM          ; --> Abfrage Stellmodus
TS3:    TCM     P1,#%10       ; Spalte 3(P14) = H ? (PRM; PR3; PR6; PR9)
        JR      NZ,LPL       ; --> nein --> letztes Programm laden
        ADD     R15,#%18     ; Programmverteiler % 0718; 071B; 071E; 0721
STM:    CP      R2,#%80      ; Stellmodus ?
        JR      Z,SM1        ; --> nein
        ADD     R15,#%48     ; ja, ( Digitprog.)Progr. -Vert. ab % 0748
SM1:    RET
;------------------------------------------------------------------------------
; Programm laut Tastatur laden
;------------------------------------------------------------------------------
PLD:    SRP     #%10
        LDC     R11,@RR14    ; Maschinencode von Programmverteiler holen
        CP      R11,#%8D     ; Sprung ? ( = JP xxx ) - PRP;PR0,PRM -
        JR      Z,PL3        ; --> ja
        CP      R11,#%AC     ; Digitalprogrammierung ? ( = LD R10,#%xx )
        JR      Z,PL4        ; --> ja
        LD      R7,R15       ; Progr. -Vert. speichern ( PR1-PR9;P11-P19 )
        CP      R2,#%80      ; Stellmodus ? ( verhindert Sprung in 2. Ebene )
        JR      NZ,PL1       ; --> ja
        CP      R7,#%24      ; Programmspeicherreg. schon in der 2. Ebene ?
        JR      C, PL1       ; --> ja
        CP      %0C,#%01     ; Zusatztaste 16ms gedrckt ?
        JR      C,PL1        ; --> nein
        ADD     R15,#%01     ; Programmverteiler in 2. Ebene
        LD      R7,R15       ; Prog. -Vert. speichern ( P11 - P91 )
PL1:    LDC     R6,@RR14     ; Daten f. Stelle 3 f. entspr. Programm holen
        INC     R15          ; Adresse um eins erhhen
        LDC     R8,@RR14     ; Daten f. Zeigerreg. Stelle 1;2;4;5
        INC     R15          ; Adresse um eins erhhen
        LDC     R9,@RR14     ; Daten f. LED-Anzeige vom Speicherplatz holen
        SUB     R15,#%02     ; Programmverteiler auf Ursprung laden
        LD      R4,@R8       ; Reg. f. St4;5 m. Wert v. gezeigerten Reg. laden
        DEC     R8           ; Zeigerregister auf Ursprung laden
        RET
PL3:    CALL    @RR14        ; Sprungverteiler f. PRP; PR0; PRM
        CP      R15,#%0C     ; Programmverteiler auf PR0 ?
        JR      NZ,LPL       ; --> nein = letztes Programm wieder laden
        LD      R7,R15       ; Adrese PR0 im Programmspeicherreg. merken
        RET
PL4:    CALL    @RR14        ; Daten f. Dekadenprogrammierung auf R10 laden
        LD      R11,@R2      ; Daten von d. neuzuladenen Reg. ins AR laden
        CP      %35,%37      ; Stelle 1 oder 4 neu zu laden ? ( = #%04 )
        JR      NZ,SWP       ; --> ja
        AND     R11,#%F0     ; Stelle 2 bzw. 5 lschen ( ST1;4 unverndert )
        OR      R11,R10      ; Stelle 2 bzw. 5 mit neuzuladenen Daten
        JR      SW1
SWP:    SWAP    R10          ; Daten zum HWT ( = #% X,0 )
        AND     R11,#%0F     ; Stelle 1 bzw. 4 lschen ( St2;5 unverndert )
        OR      R11,R10      ; Stelle 1 bzw. 4 mit neuzuladenen Daten
SW1:    LD      @R2,R11      ; neugeladenen Daten zum gezeigertem Register
LPL:    LD      R15,R7       ; letztes Abzeigeprogr. wieder laden !
        RET
;------------------------------------------------------------------------------
; Registerkorrektur
;------------------------------------------------------------------------------
RKO:    SRP     #%40
        CP      R5,#%60      ; Weckwiederholungszeit f. Wecker 1 < 59 min ?
        JR      C,RK1        ; --> nein
        LD      R5,#59       ; ja, begrenzen auf 59 min !
RK1:    CP      R7,#%60      ; Weckwiederholungszeit f. Wecker 2 > 59 min ?
        JR      C,RK2        ; --> nein
        LD      R7,#%59      ; ja, begrenzen auf 59 min !
RK2:    CP      %52,#%00     ; Zeitdauer f. Uhrzeit > 00 s ?
        JR      NZ,RK3       ; --> ja
        CP      %53,#%00     ; nein, Zeitdauer f. Temp./Datum > 00 s ?
        JR      NZ,RK3       ; --> ja
        LD      %52,#%10     ; nein, Zeitdauer f. Uhrzeit = 10 s !
RK3:    CP      R8,#%24      ; Beginn Stundenton > 23 h ?
        JR      C,RK4        ; --> nein
        CLR     R8           ; ja, Beginn Stundenton = 0 h !
RK4:    RET
;------------------------------------------------------------------------------
; Dekadenprogrammierung
;------------------------------------------------------------------------------
PRP:    TM      P0,#%80      ; Zusatztaste gedrckt ?
        JR      Z,P6P        ; --> ja = vorzeitiges Verlassen d. Stellmodus
        CP      %2C,#%19     ; 250ms erreicht ?
        JR      C,P1P        ; --> nein
        CLR     %2C          ; Abtastregister lschen
        CP      R2,#%80      ; im Stellmodus gewesen ?
        JR      NZ,P2P       ; --> ja
        LD      R2,R8        ; nein, Zeigerreg. f. St1;2 speichern
        LD      %35,#%04     ; Dezimalpunkt (Dp.) f. St1 laden
P1P:    RET                  ;    ( = Programmierung d. 1. Dekade (St1) )
P2P:    CP      %35,#%04     ; Dp. in der 1. Stelle ?
        JR      NZ,P3P       ; --> nein
        CLR     %35          ; ja, Dp. St1 lschen
        LD      %36,#%04     ; Dp. f. St2 laden ( Progr. d. 2. Dekande (St2) )
        RET
P3P:    CP      %36,#%04     ; Dp. in der 2. Stelle ?
        JR      NZ,P4P       ; --> nein
        CLR     %36          ; ja, Dp. St2 lchen
        LD      %37,#%04     ; Dp. f. St4 laden ( = Progr. d. 4. Dekande ( St4 )
        INC     R2           ; Speicher f. Zeigerregisten auf ST4;5
        RET
P4P:    CP      %37,#%04     ; Dp. in der 4. Stelle ?
        JR      NZ,P5P       ; --> nein = Stellmodus verlassen
        CLR     %37          ; ja, Dp. St4 lschen
        LD      %38,#%04     ; Dp. f. St5 laden ( = Progr. d. 5. Dekade (St5) )
        RET
P6P:    CLR     %35          ; Dp. von St1 oder 2 oder 4 oder 5 lchen
        CLR     %36
        CLR     %37          ; - verlassen des Stellmodus -
P5P:    CLR     %38          ;
        LD      R2,#%80      ; Speicher f. Zeigerreg. auf #%80
        RET                  ; ( = Stellmodus aus )
;------------------------------------------------------------------------------
; Weckeraktivierung
;------------------------------------------------------------------------------
PRM:    CP      %2C,#%19     ; 250 ms erreicht ?
        JR      C,P1M        ; --> nein
        CLR     %2C          ; Abtastregister lschen
        LD      R11,%3F      ; AR <-- Reg. f. Weckeraktivierung
        AND     %3F,#%9F     ; Bit 5;6 = L ( WE1; WE2 )
        ADD     R11,#%20     ; 20 + 20 = 40 + 20 = 60 + 20 = 80 + 20 = A0
        AND     R11,#%60     ; bur 5;6   40        60        00        20
        OR      %3F,R11      ; zurckladen
P1M:    RET
;------------------------------------------------------------------------------
; Anzeigenprogramm fr Uhrzeit; Temperatur; Datum
;------------------------------------------------------------------------------
PR0:    LD      R8,#%25      ; Zeigerreg. f. Anzeigedatan auf Reg. f. min
        LD      R9,%3F       ; LED-Reg mit Weckeraktivierung kaden
        CP      R2,#%80      ; Stellmodus ?
        JR      NZ,P01       ; --> ja = Anzeige Uhrzeit
        TM      P0,#%80      ; Zusatztate gedrckt ?
        JR      Z,P06        ; --> ja = Anzeige Minuten u. Sekunden
        CP      %34,#%01     ; Sprungverteiler auf Uhrzeit ? ( = #%01 )
        JR      NZ,P02       ; --> nein
        CP      %2E,#%00     ; ja Zeit f. Uhrenzeitanzeige vorber ?
        JR      NZ,P01       ; --> nein = Anzeige Uhrzeit
        LD      %34,#%02     ; ja, prungverteiler auf Temperaturanzeige
        LD      R11, %53     ; AR <-- Zeitdauer f. Temp. -Anz. laden
        SWAP    R11          ; Datum, Temperatur
        AND     R11,#%0F     ; nur Temperaturdauer
        LD      %2E,R11      ; Abtastreg. f. Wechselzeit mit Zeitdauen Temp.
        RET
;-----------------------------
; Anzeige der Uhrzeit
;-----------------------------
P01:    LD      R4,%25       ; Reg. f. St1;2 mit Reg. f. Minuten laden
        LD      R5,%26       ; Reg. f. St4;5 mit Reg. f. Stunden laden
        LD      R6,#%EF      ; Reg. f. St3 = Doppelpunkt im 1Hz-Rhytmus
        RET
P02:    CP      %34,#%02     ; Sprungverteiler auf Temperatur ? ( = #%02 )
        JR      NZ,P04       ; --> nein
        CP      %2E,#%00     ; ja, Zeit f. Temperaturanzeige vorber ?
        JR      NZ,P03       ; --> nein = Anzeige Temperatur
        LD      %34,#%03     ; ja, Sprungverteiler auf Datumanzeige
        LD      R11,%53      ; AR <-- Zeitdauer f. Datumanzeige laden
        AND     R11,#%0F     ; nur Datumdauer
        LD      %2E,R11      ; Abtastreg. f. Wechselzeit mit Zeitdauer Datum
        RET
;-----------------------------
; Anzeige der Temparatur
;-----------------------------        
P03:    LD      R4,%33       ; Reg. f. St1;2 mit Reg. f. T(IST) laden
        LD      R5,#%BA      ; Reg. f. St4;5 mit grad Celsius laden
        LD      R6,%32       ; Reg. f. St3 mit Vorzeichen T(IST) laden
        RET
P04:    CP      %2E,#%00     ; Zeit f. Datumanzeige vorber ?
        JR      NZ,P05       ; --> nein = Anzeige Datum
        LD      %34,#%01     ; ja, Sprungverteiler auf Uhrzeitanzeige
        LD      %2E,%52      ; Abtastreg. f. Wechselzeit mit Zeitdauer Uhrz.
        RET
;-----------------------------
; Anzeige des Datum
;-----------------------------
P05:    LD      R4,%27       ; Reg. f. ST1;2 mit Reg.f. Tage laden
        LD      R5,%28       ; Reg. f. St4;5 mit Reg. f. Monate laden
        LD      R6,#%FD      ; Reg. f. St3 mit #%FD laden ( = Anzeige d. Dp.
        RET                  ; in ST2;5 f. Datumanzeige; St3 ist inaktiv )
;-----------------------------
; Anzeige der Minuten + Sekunden
;-----------------------------
P06:    LD      R4,%26       ; Reg. f. St1;2 mit Reg. f. Minuten laden
        LD      R5,%22       ; Reg. f. St4;5 mit Reg. f. Sekunden laden
        LD      R6,#%FE      ; Reg. f. St3 mit stndigen Doppelpunkt laden
        RET
;------------------------------------------------------------------------------
; Weckzeit 1 mit Pausenzeit + Weckwiederholung
;------------------------------------------------------------------------------        
WE1:   CLR     RP           ; - Sprungverteiler f. Wecker 1 -
       CP      R5,#%01      ; Weckmelodie zu Ende und wieder neu laden ?
       JR      Z,W11        ; --> ja
       CP      R5,#%02      ; Weckwiederholung aktiviert worden ?
       JR      Z,W12        ; --> ja
       CP      R5,#%03      ; Weckwdhlg. ausgeschaltet, Abschaltton noch ?
       JR      Z,W13        ; --> ja
       CP      R5,#%04      ; Weckmelodie mit Pausenzeit aktiv ?
       JR      Z,W14        ; --> ja
       CP      R5,#%05      ; Wecker aus, aber Uhrzeit = Weckzeit noch ?
       JR      Z,W15        ; --> ja
       TCM     %3F,#%40     ; Wecker 1 im Weckeraktivierungsreg. aktiv ?
       JR      NZ,W16       ; --> nein
       CP      %40,%25      ; ja, Weckzeit (h) = Uhrzeit (h) ?
       JR      NZ,W16       ; --> nein
       CP      %41,%26      ; ja, Weckzeit (min) = Uhrzeit (min) ?
       JR      NZ,W16       ; --> nein
W11:   LD      R5,#%04      ; ja, Sprungvert. auf Ablauf Weckmelodie
       LD      R6,#%07      ; Anfangsadresse f. Weckmelodie 1 laden
       LD      R7,#%6C      ; ab %076C
       LD      PRE1,WE1_PRE1 ; Vorteilerreg. = 3
W14:   CP      R12,#%01     ; Zusatztaste 16ms bettigt ?
       JR      C,W17        ; --> nein = keine Weckwiederholung
       LD      R5,#%02      ; ja, Sprungvert. auf Weckwiederholung aktiv
       AND     %3F,#%BF     ; LED f. WE1 im PR0 lschen
       OR      %3F,#%04     ; LED f. WE1 blinkt im PR0 ( Weckwdhlg. aktiv )
       CALL    TMA          ; --> Ton abschalten ( T1 gesperrt )
       LD      %3D,%26      ; Reg. f. Weckwdhlg. -Dauer = Uhrzeit ( min )
       ADD     %3D,%45      ; + programmierter Weckwdhlg. ( min )
       DA      %3D          ; Dezimalkorrektur ( bis 118 min )
       JR      C,W18        ; > 99 min ? --> ja
       CP      %3D,#%60     ; nein, > 59 min ?
       JR      C,W12        ; --> nein
W18:   SUB     %3D,#%60     ; ja berlaufkorrektur ( von 60 bis 118 )
       DA      %3D          ; ( = 00 bis 58 min )
W12:   CP      R12,#%05     ; Zusatztaste 1,04s = bettigt ?
       JR      C,W19        ; --> nein = Weckwdhlg. bleibt aktiv
       LD      R5,#%03      ; ja, Sprungvert. auf Weckerabschaltung
       CLR     %2F          ; Reg. fr Tonlnge = 0
       LD      T1,#%80      ; f=f(INT)/PRE1*T1 = 625KHz/3*128 = 1,68KHz
       OR      TMR,#%8C     ; T1 auf P36; T1 + PRE1 laden
       JR      W1D          ; --> LED f. WE1 wieder setzen
W13:   CP      %2F,#%30     ; 300ms Piepton f. Weckerabschaltung corbei ?
       JR      C,W1A        ; --> nein
       LD      R5,#%05      ; ja, Sprungvert. auf Abfrage WE1 = Uhr noch
       CALL    TMA          ; --> Ton abschalten ( T1 gesperrt )
W15:   CP      %41,%26      ; Abfr. Weckzeit = Uhrzeit ?(Bei Weckerabsch.)
       JR      Z,W1A        ; --> ja
       CLR     R5           ; nein, Sprungverteiler inaktiv
       RET                  
W16:   JR      W1A          ; ( Einsparung von "Jump's" )
W17:   CP      %2D,#%00     ; Pausenzeit Weckmelodie 1/2 aktiv ?
       JR      NZ,W1A       ; --> ja
       CP      %2F,#%12     ; nein, Abtastreg. f. Tonlnge 120ms erreicht ?
       JR      C,W1A        ; --> nein
       CLR     %2F          ; ja, Abtastreg. lschen
       CP      R7,#%A4      ; Melodieende erreicht ? ( %07A3 H )
       JR      NZ,W1B       ; --> nein = nchsten Ton laden
       LD      R5,#%01      ; ja, Sprungverteiler auf Melodie neu laden
       LD      %2D,%44      ; Abtastreg. mit progr. Pausenzeit f. WE1 laden
       CALL    TMA          ; --> Ton abschalten ( entspr. Pausenzeit )
       RET
W1B:   LDC     R15,@RR6     ; Teilerwert f. Melodiefrequenz von Tab. holen
       LD      T1,R15       ; T1 laden ( 0 bis 256 )
       INC     R7           ; Melodieadresse +1 H
       TCM     TMR,#%80     ; TMR schon aktiviert worden ?
       JR      Z,W1C        ; --> ja
       OR      TMR,#%8C     ; nein Tonfrequenz auf P36 und T1, PRE1 laden
W1C:   RET
W19:   CP      %26,%3D      ; Weckerwiederholung vorbei ? ( Uhr=WE1+WWDHLGn )
       JR      NZ,W1A       ; --> nein
       LD      R5,#%01      ; ja, Sprungverteiler auf Melodie neu laden
W1D:   AND     %3F,%FB      ; LED f. WE1 stndig aktiv
       OR      %3F,#%40     ; LED f. WE1 ohne blinken
W1A:   RET

;------------------------------------------------------------------------------
; Weckzeit 2 mit Pausenzeit + Weckwiederholung
;------------------------------------------------------------------------------        
WE2:   CLR     RP           ; - Sprungverteiler f. Wecker 1 -
       CP      R4,#%01      ; Weckmelodie zu Ende und wieder neu laden ?
       JR      Z,W21        ; --> ja
       CP      R4,#%02      ; Weckwiederholung aktiviert worden ?
       JR      Z,W22        ; --> ja
       CP      R4,#%03      ; Weckwdhlg. ausgeschaltet, Abschaltton noch ?
       JR      Z,W23        ; --> ja
       CP      R4,#%04      ; Weckmelodie mit Pausenzeit aktiv ?
       JR      Z,W24        ; --> ja
       CP      R4,#%05      ; Wecker aus, aber Uhrzeit = Weckzeit noch ?
       JR      Z,W25        ; --> ja
       TCM     %3F,#%20     ; Wecker 1 im Weckeraktivierungsreg. aktiv ?
       JR      NZ,W26       ; --> nein
       CP      %42,%25      ; ja, Weckzeit (h) = Uhrzeit (h) ?
       JR      NZ,W26       ; --> nein
       CP      %43,%26      ; ja, Weckzeit (min) = Uhrzeit (min) ?
       JR      NZ,W26       ; --> nein
W21:   LD      R4,#%04      ; ja, Sprungvert. auf Ablauf Weckmelodie
       LD      R6,#%07      ; Anfangsadresse f. Weckmelodie 1 laden
       LD      R7,#%A4      ; ab %07A4
       LD      PRE1,WE2_PRE1 ; Vorteilerreg. = 3
W24:   CP      R12,#%01     ; Zusatztaste 16ms bettigt ?
       JR      C,W27        ; --> nein = keine Weckwiederholung
       LD      R4,#%02      ; ja, Sprungvert. auf Weckwiederholung aktiv
       AND     %3F,#%DF     ; LED f. WE1 im PR0 lschen
       OR      %3F,#%02     ; LED f. WE1 blinkt im PR0 ( Weckwdhlg. aktiv )
       CALL    TMA          ; --> Ton abschalten ( T1 gesperrt )
       LD      %3E,%26      ; Reg. f. Weckwdhlg. -Dauer = Uhrzeit ( min )
       ADD     %3E,%47      ; + programmierter Weckwdhlg. ( min )
       DA      %3E          ; Dezimalkorrektur ( bis 118 min )
       JR      C,W28        ; > 99 min ? --> ja
       CP      %3E,#%60     ; nein, > 59 min ?
       JR      C,W22        ; --> nein
W28:   SUB     %3E,#%60     ; ja berlaufkorrektur ( von 60 bis 118 )
       DA      %3E          ; ( = 00 bis 58 min )
W22:   CP      R12,#%05     ; Zusatztaste 1,04s = bettigt ?
       JR      C,W29        ; --> nein = Weckwdhlg. bleibt aktiv
       LD      R4,#%03      ; ja, Sprungvert. auf Weckerabschaltung
       CLR     %2F          ; Reg. fr Tonlnge = 0
       LD      T1,#%80      ; f=f(INT)/PRE1*T1 = 625KHz/3*128 = 1,68KHz
       OR      TMR,#%8C     ; T1 auf P36; T1 + PRE1 laden
       JR      W2D          ; --> LED f. WE1 wieder setzen
W23:   CP      %2F,#%30     ; 300ms Piepton f. Weckerabschaltung corbei ?
       JR      C,W2A        ; --> nein
       LD      R4,#%05      ; ja, Sprungvert. auf Abfrage WE1 = Uhr noch
       CALL    TMA          ; --> Ton abschalten ( T1 gesperrt )
W25:   CP      %43,%26      ; Abfr. Weckzeit = Uhrzeit ?(Bei Weckerabsch.)
       JR      Z,W2A        ; --> ja
       CLR     R5           ; nein, Sprungverteiler inaktiv
       RET                  
W26:   JR      W2A          ; ( Einsparung von "Jump's" )
W27:   CP      %2D,#%00     ; Pausenzeit Weckmelodie 1/2 aktiv ?
       JR      NZ,W2A       ; --> ja
       CP      %2F,#%10     ; nein, Abtastreg. f. Tonlnge 120ms erreicht ?
       JR      C,W2A        ; --> nein
       CLR     %2F          ; ja, Abtastreg. lschen
       CP      R7,#%E4      ; Melodieende erreicht ? ( %07E3 H )
       JR      NZ,W2B       ; --> nein = nchsten Ton laden
       LD      R4,#%01      ; ja, Sprungverteiler auf Melodie neu laden
       LD      %2D,%46      ; Abtastreg. mit progr. Pausenzeit f. WE1 laden
       CALL    TMA          ; --> Ton abschalten ( entspr. Pausenzeit )
       RET
W2B:   LDC     R15,@RR6     ; Teilerwert f. Melodiefrequenz von Tab. holen
       LD      T1,R15       ; T1 laden ( 0 bis 256 )
       INC     R7           ; Melodieadresse +1 H
       TCM     TMR,#%80     ; TMR schon aktiviert worden ?
       JR      Z,W2C        ; --> ja
       OR      TMR,#%8C     ; nein Tonfrequenz auf P36 und T1, PRE1 laden
W2C:   RET
W29:   CP      %26,%3E      ; Weckerwiederholung vorbei ? ( Uhr=WE1+WWDHLGn )
       JR      NZ,W2A       ; --> nein
       LD      R4,#%01      ; ja, Sprungverteiler auf Melodie neu laden
W2D:   AND     %3F,%FD      ; LED f. WE1 stndig aktiv
       OR      %3F,#%20     ; LED f. WE1 ohne blinken
W2A:   RET
;------------------------------------------------------------------------------
; 2 E/A - Zeiten mit Datum
;------------------------------------------------------------------------------
REL:   TCM     %4C,#%F0     ; Datum fr die 1. Einschaltzeit programmiert ?
       JR      Z,R1E        ; --> nein ( St1 = #%F0 d.h. Anzeige dunkel )
       CP      %4C,%27      ; ja, 1. Einschaltzeit = Datum (d) ?
       JR      NZ,RA1       ; --> nein
       CP      %4D,%28      ; ja, 1. Einschaltzeit = Datum (m) ?
       JR      NZ,RA1       ; --> nein
R1E:   CP      %4A,%25      ; ja, 1. Einschaltzeit = Uhrzeit (h)
       JR      NZ,RA1       ; --> nein
       CP      %4B,%26      ; ja, 1. Einschaltzeit = Uhrzeit (min) ?
       JR      Z,EIN        ; --> ja = Relais einschalten !
       
RA1:   TCM     %50,#%F0     ; Datum fr die 1. Ausschaltzeit programmiert ?
       JR      Z,R1A        ; --> nein ( St1 = #%F0 d.h. Anzeige dunkel )
       CP      %50,%27      ; ja, 1. Ausschaltzeit = Datum (d) ?
       JR      NZ,RE2       ; --> nein
       CP      %51,%28      ; ja, 1. Ausschaltzeit = Datum (m) ?
       JR      NZ,RE2       ; --> nein
R1A:   CP      %4E,%25      ; ja, 1. Ausschaltzeit = Uhrzeit (h)
       JR      NZ,RE2       ; --> nein
       CP      %4F,%26      ; ja, 1. Ausschaltzeit = Uhrzeit (min) ?
       JR      Z,AUS        ; --> ja = Relais ausschalten !

RE2:   TCM     %58,#%F0     ; Datum fr die 2. Einschaltzeit programmiert ?
       JR      Z,R2E        ; --> nein ( St1 = #%F0 d.h. Anzeige dunkel )
       CP      %58,%27      ; ja, 2. Einschaltzeit = Datum (d) ?
       JR      NZ,RA2       ; --> nein
       CP      %59,%28      ; ja, 2. Einschaltzeit = Datum (m) ?
       JR      NZ,RA2       ; --> nein
R2E:   CP      %56,%25      ; ja, 2. Einschaltzeit = Uhrzeit (h)
       JR      NZ,RA2       ; --> nein
       CP      %57,%26      ; ja, 2. Einschaltzeit = Uhrzeit (min) ?
       JR      Z,EIN        ; --> ja = Relais einschalten !

RA2:   TCM     %5C,#%F0     ; Datum fr die 2. Ausschaltzeit programmiert ?
       JR      Z,R2A        ; --> nein ( St1 = #%F0 d.h. Anzeige dunkel )
       CP      %5C,%27      ; ja, 2. Ausschaltzeit = Datum (d) ?
       JR      NZ,RE3       ; --> nein
       CP      %5D,%28      ; ja, 2. Ausschaltzeit = Datum (m) ?
       JR      NZ,RE3       ; --> nein
R2A:   CP      %5A,%25      ; ja, 2. Ausschaltzeit = Uhrzeit (h)
       JR      NZ,RE3       ; --> nein
       CP      %5B,%26      ; ja, 2. Ausschaltzeit = Uhrzeit (min) ?
       JR      NZ,RE3       ; --> nein
AUS:   OR      P3,#%10      ; --> ja = Relais ausschalten ( P34 = H )
       AND     %3F,#%7F     ; berwachungs-LED im PR0 lschen
       AND     FLAGS,#%FD   ; Bit 1 Temperaturreglerzeit inaktiv
RE3:   RET
EIN:   TCM     FLAGS,#%02   ; Temperraturregleizeit aktiv ?
       JR      Z,EI1        ; --> ja (Relais wird vom Regler geschaltet )
       AND     P3,#%E0      ; nein, Relais einschalten ( P34 = L )
       OR      %3F,#%80     ; berwachungs-LED im PR0 einschalten
       OR      FLAGS,#%02   ; Temperarurreglerzeit aktivieren
EI1:   RET

;------------------------------------------------------------------------------
; 4 E/A - Zeiten ohne Datum ( spter )
;------------------------------------------------------------------------------


;------------------------------------------------------------------------------
; Ausgabe des Stundentones
;------------------------------------------------------------------------------
TON:   SRP     #%20
       LD      R9,%48       ; AR <-- Beginnzeit Stundenton ( TON an )
TO1:   LD      R10,%25      ; AR <-- Uhrzeit (h)
       CP      R9,R10       ; TON an + n h = Uhrzeit (h) ?
       JR      Z,TO2        ; --> ja = Stundenton aktivieren
       ADD     R9,#%01      ; nein, TON an um 1h erhhen
       DA      R9           ; BCD Format
       CP      R9,#%24      ; TON an > 23 h ?
       JR      C,TO3        ; --> nein
       CLR     R9           ; ja, TON an = 00h
TO3:   LD      R10,%49      ; AR <-- Endzeit Stundenton ( TON ab )
       ADD     R10,#%01     ; TON ab um 1h erhhen
       DA      R10          ; BCD Format
       CP      R10,#%24     ; TON ab > 23h
       JR      C,TO4        ; --> nein
       CLR     R10          ; ja, TON ab = 00h
TO4:   CP      R9,R10       ; TON an + n h = TON ab + 1h
       JR      NZ,TO1       ; --> nein = TON an + 1h
       RET                  ; --> ja = Stundenton inaktiv
TO2:   CP      R10,R4       ; Uhrzeit (h) = Merker Uhrzeit (h) ?
       JR      NZ,TO6       ; --> nein = Stundenton aktivieren
       CP      R3,#%00      ; ja, aber Ton noch aktiv ?
       JR      Z,TO7        ; --> nein
TO6:   CP      R15,#%30     ; Abtastreg. f. Tonlnge 300ms erreicht ?
       JR      C,TO7        ; --> nein
       CP      R3,#%F0      ; ja, alle 3 Frequenzen durchlaufen ?
       JR      NC,TO8       ; --> ja
       CLR     R15          ; nein, Abtastreg. f. Tonlnge = 0
       ADD     R3,#%50      ; Teilwert f. Frequenz laden
       LD      T1,R3        ; /80(1953 Hz)-->/160(977Hz)-->/240(651Hz)
       TCM     TMR,#%80     ; Timer zum Ausgang P36 schon geladen ?
       JR      Z,TO7        ; --> ja
       LD      PRE1,TON_PRE1 ; nein, Vorteiler mit 4 laden ( = 156,25KHz )
       OR      TMR,#%8C     ; T1 --> P36; T1 und PRE1 laden
TO7:   RET
TO8:   CALL    TMA          ; Ton abschalten
       LD      R4,R10       ; Merk. Uhrzeit(h)=Uhrzeit(h) nach Tonausgabe
       RET
;------------------------------------------------------------------------------
; Temperaturregler mit Hyst
;------------------------------------------------------------------------------
RGL:   SRP     #%30
MSD:   TM      P0,#%20      ; P05 = L ( MSD ) ?
       JR      NZ,NSD       ; --> nein
       LD      R11,P0       ; AR <-- MSD, Digitwer
       AND     R11,#%0F     ; nur MSD-Wert
       LD      R9,R11       ; Zsp. MSD <-- 0, MSD
       OR      R12,#%20     ; merken Digit MSD eingelesen
NSD:   TM      P0,#%10      ; P4 = L ( NSD ) ?
       JR      NZ,LSD       ; --> nein
       LD      R11,P0       ; AR <-- NSD, Digitwert
       AND     R11,#%0F     ; nur NSD-Wert
       SWAP    R11          ; NSD, 0
       AND     R10,#%0F     ; Zsp. 0 ,LSD
       OR      R10,R11      ; Zsp. NSD,LSD <-- NSD, 0
       OR      R12,#%10     ; merken Digit NSD eingelesen
LSD:   TM      P0,#%40      ; P06 = L ( LSD )
       JR      NZ,RG0       ; --> nein
       LD      R11,P0       ; AR <-- LSD, Digitwert
       AND     R11,#%0F     ; nur LSD-Wert
       AND     R10,#%F0     ; Zsp. NSD, 0
       OR      R10,R11      ; Zsp. NSD,LSD <-- 0, LSD
       OR      R12,#%40     ; merken Digit LSD eingelesen
RG0:   TCM     R12,#%70     ; Digitstellen MSD,NSD,LSD eingelesen ?
       JR      NZ,RG2       ; --> nein
       CLR     R12          ; ja, Merkreg. f. Digitstellen rcksetzen
       CP      R1,R10       ; T(IST) MSD-1 = T(LST) MSD 0 ?
       JR      NZ,RG6       ; --> nein = Z(IST) -1 <-- T(IST) 0
       CP      R0,R9        ; T(IST) NSD,LSD -1 = T(IST) NSD,LSD 0 ?
       JR      NZ,RG6       ; --> nein = T(IST) -1 <-- T(IST) 0
RG3:   SUB     R1,#%73      ; ja, T(IST) -1 = T(IST) 0
       DA      R1
       SBC     R0,#%02      ; T(IST) - 0273 mV ( = 0 C ) Crad
       DA      R0           ; ( z.B. 0258 mV - 0273 mV = 9985 )
       LD      R2,#%FF      ; Daten f. Stelle 3 = Anzeige aus
       JR      NC,RG4       ; Ergebnis pos. ? --> ja ( d.h. T(IST) > 0 C ) Grad
       LD      R3,R1        ; nein, d.h. Temperatur negativ !
       CLR     R1           ; AR <-- T(IST) - 0273 mV ( z.B. 9985 )
       SUB     R1,R3        ; 00 - AR( T(IST) - 0273 mV )
       DA      R1           ; ( z.B. 00 - 85 = 15 C ) Grad
       LD      R2,#%FD      ; Daten f. Stelle 3 = Minuszeichen aktiv
RG4:   LD      R3,R1        ; AR <-- T(IST) ( in C ) Grad 00 bis 99
       TM      FLAGS,#%02   ; Temperaturregelzeit aktiv ?
       JR      Z,RG6        ; --> nein = Ursprungswert wieder laden
       TCM     %54,#%F0     ; ja, T(SOLL) programmiert ( = Regler aktiv )
       JR      Z,RG6        ; --> nein = Ursprungswert wieder laden
       CP      R2,#%F0      ; ja, Temeratur negativ ?
       JR      Z,RAN        ; --> ja = Relais einschalten
       SUB     R1,%54       ; ABW = T(IST) - T(SOLL)
       DA      R1
       JR      PL,RG5       ; ABW = positiv ? --> ja, d. h. T(IST) > T(SOLL)
       ADD     R1,%55       ; nein, ABW + HYS ( ABW negativ ! )
       DA      R1
       JR      PL,RG6       ; Ergebnis pos. ? --> ja, d.h. ABW innerh. HYST.
RAN:   AND     P3,#%E0      ; nein, Relais einschalten ( P34 = L )
       OR      R15,#%80     ; berwachungs-LED im PR0 stndig aktiv
       JR      RG6          ; --> Ursprungswert wieder laden
RG5:   SUB     R1,%55       ; ABW + HYS ( ABW positiv ! )
       DA      R1
       JR      MI,RG6       ; Ergebnis neg. ? --> ja d.h. ABW innerh. HYST.
RZU:   OR      P3,#%10      ; nein, Relais ausschalten ( P24 = H )
       AND     R15,#%7F     ; berwachungs-LED im PR0 inaktiv
RG6:   LD      R1,R10       ; Ursprungswert beim Einlesen wieder laden !
       LD      R0,R9        ; T(IST) -1 = T(IST)
RG2:   RET
;------------------------------------------------------------------------------
; Datumgenerierung
;------------------------------------------------------------------------------
DAT:   SRP     #%20
       LD      R10,#%07     ; Anfangsadresse f. Abfrage der Monatsdaten
       LD      R11,#%E4     ; %07E4 H
       CP      R8,#%10      ; Reg. f. Monate > 9 ( notwendig f. Adressierung )
       JR      C,DA1        ; --> nein
       SUB     R8,#%06      ; ja, 10 bis 12h - 6 h = 0A bis 0C h
DA1:   ADD     R11,R8       ; Anfangsadresse + Monat ( 01 bis 0C h
       LDC     R11,@RR10    ; AR <-- Monatsende ( 28;30;31 ) %07E5 - %07F0 H
       CP      R11,R7       ; Monatsende > Reg. f. Tage ?
       JR      NC,DA2       ; --> ja
       LD      R7,#%01      ; nein, Reg. f. Tage = Monatsanfang
       INC     R8           ; Reg. f. Monate +1 ( d.h. nchster Monat )
       CP      R8,#%0D      ; Dezember berschritten ? ( > 0C h )
       JR      C,DA2        ; --> nein
       LD      R8,#%01      ; ja, = Jannuar ! ( Reg. f. Monate = 01 )
DA2:   ADD     R8,#%00      ; Dezimalkorrektor
       DA      R8           ; aus 0A - 0C h wird 10 - 12 !
       RET
;------------------------------------------------------------------------------
; Kurzzeitwecker
;------------------------------------------------------------------------------
KZW:   SRP     #%50
       TCM     %3F,#%01     ; Start schon erfolgt ?
       JR      Z,KZ1        ; --> ja = Abfrage Zeit vorber ?
       TCM     %3F,#%10     ; nein, Piepton aber noch aktiv ?
       JR      Z,KZ2        ; --> ja = Abfrage Abtastreg. f. Tonlnge
       CP      %18,#%5E     ; Programm KZW auf Anzeige ?
       JR      NZ,KZ4       ; --> nein
       CP      %0C,#%00     ; ja, Zusatztaste f. 2. Ebene losgelassen ?
       JR      NZ,KZ4       ; --> nein
       CP      %0D,#%F2     ; ja, und f. Start nochmals bettigt ( > 2ms )
       JR      C,KZ4        ; --> nein = Anzeige programmierter KZW
       LD      %0E,R14      ; ja, Start !
       LD      %0F,R15      ; Tsp. KZW <-- Programmierter KZW
       OR      %3F,#%01     ; Merker f. Start erfolgt setzen ( Bit 0 = 1 )
KZ1:   CP      R14,#%00     ; Reg. f. KZW ( min ) = 0 ?
       JR      NZ,KZ4       ; --> nein = Anzeige aktueller Zhlerstand
       CP      R15,#%00     ; ja, Reg. f. KZW (S) = 0 ?
       JR      NZ,KZ4       ; --> nein = Anzeige aktueller Zhlerstand
       AND     %3F,#%FE     ; ja, Kurzzeitwecker inaktiv setzen
       OR      %3F,#%10     ; Merker f. Piepton aktiv setzen
       CLR     %2F          ; Abtastreg. f. Tonlnge rcksetzen
       LD      T1,#%80      ; T1 mit 128 laden
       LD      PRE1,KZW_PRE1    ; Vorteiler mit 3 = f(int)/3*128 = 1628Hz
       OR      TMR,#%8C     ; T1 auf P36; T1 und PRE1 laden
KZ2:   CP      %2F,#%95     ; Abtastreg. f. Tonlnge 950ms erreicht ?
       JR      C,KZ4        ; --> nein = Anzeige 00:00
       AND     %3F,#%EF     ; ja, Merker "Piepton inaktiv" laden
       LD      R14,%0E      ; Anzeige KZW <-- Zsp. KZW
       LD      R15,%0F      ; ( Ursprungswert erscheint auf der Anzeige )
       CALL    TMA          ; Ton abschalten
KZ4:   RET
;------------------------------------------------------------------------------
; Stopuhr mit Zwischenhalt
;------------------------------------------------------------------------------
SPU:   CLR     RP
       CP      %18,#%08     ; Stopuhr auf Anzeige ( Zeiger auf %08 )
       JR      NZ,SP1       ; --> nein = Stopuhr inaktiv setzen
       CP      %55,#%99     ; ja, Hysterese d. Temperaturreglers auf 99 ?
       JR      NZ,SP2       ; --A nein = Stopuhrbetrieb
       LD      R8, %39      ; ja, ADU-Wert in mV auf der Anzeige f. Stopuhr
       LD      R9, %3A      ; 
SP1:   AND     FLAGS,#%FE   ; Stopuhr inaktiv setzen ( Bit 0 = 0 )
       RET
;---------------------------
; Stopuhrbetrieb
;---------------------------      
SP2:   TM      FLAGS,#%01   ; Stopuhr schon aktiv ?
       JR      Z,SP3        ; --> nein = Abfrage der Zusatztaste
       CP      R12,#%01     ; ja, Abtastreg. f. Zusatztaste 16ms erreicht ?
       JR      C,SP5        ; --> nein = aktueller Zhlerstand auf Anz.
       RET
SP3:   CP      R12,#%00     ; Zusatztaste f. 2. Ebene losgelassen ?
       JR      NZ,SP4       ; --> nein = Anzeige 00:00
       CP      R13,#%F2     ; ja und fr 2ms wieder bettigt ?
       JR      C,SP4        ; --> nein = Anzeige 00:00
       OR      FLAGS,#%01   ; ja, Start Stopuhr ( Bit 0 = 1 )
SP4:   LD      R10,%22      ; Zsp. Uhrzeit <-- Uhrzeit ( s )
       LD      R11,%21      ; "   "        <--    "    (1/100s)
SP5:   LD      R8,%22       ; Anz. Stopuhr <--    "    ( s )
       LD      R9,%21       ; "   "        <--    "    (1/100s)
       SUB     R9,R11       ; Anzeige = Uhrzeit - Zsp. Uhrzeit
       DA      R9           ; (08:19 - 25:13 = 83:06(C=1)+60:00 = 43:06)
       SBC     R8,R10       ; (48:06 - 25:13 = 22:93(C=0)       = 22:93)
       DA      R8           ; (59:99 - 25:13 = 34:86(C=0)       = 34:86)
       JR      NC,SP6       ; Uhrzeit < Zsp. Uhrzeit ? --> nein
       ADD     R8,#%60      ; ja, = Minutenkorrektur
       DA      R8           ; BCD-Korrektur
SP6:   RET
;------------------------------------------------------------------------------
; Segmentcodetabelle
;------------------------------------------------------------------------------


;           Hexacode       Zei/Zahl   Stelle 3
       .org %06D0
SEG:
       .byte   %FA          ; 0
       .byte   %12          ; 1
       .byte   %AB          ; 2
       .byte   %9B          ; 3
       .byte   %53          ; 4
       .byte   %D9          ; 5
       .byte   %F9          ; 6
       .byte   %92          ; 7
       .byte   %FB          ; 8
       .byte   %DB          ; 9
       .byte   %E8          ; Celsius
       .byte   %C3          ; Grad
       .byte   %51          ; -        Plus
       .byte   %00          ; - keine Anzeige -
       .byte   %50          ; Doppelpunkt
       .byte   %00          ; - keine Anzeige -
;       .org %06DF
;------------------------------------------------------------------------------
; Individuelle Daten
;------------------------------------------------------------------------------

       .org %06E0  ;  Format        Programmbezeichnung
IDV:    DB 5,30    ;  ( h : min )     1. Weckzeit
        DB 6,30    ;  ( h : min )     2. Weckzeit
		    DB 3,5     ;  ( s : min )  Pausenzeit + Weckerwiederholung WE1
		    DB 3,5     ;  ( s : min )  Pausenzeit + Weckerwiederholung WE2
		    DB 12,19   ;  ( h : h   ) Beginn - und Endzeit des Stundentones
		    DB 0,0     ;  ( h : min ) 1. Einschaltzeit
		    DB 0,0     ;  ( d : m   ) mit Datum
		    DB 0,0     ;  ( h : min ) 1. Ausschaltzeit
		    DB 0,0     ;  ( d _ m   ) mit Datum
		    DB 30,5,5  ;  ( s : s : s ) Anzeigenwechsel im PR0 ( Uhr;Temp;Datum)
		    DB 0,0     ;  ( C : C )   Temperaturregler mit Hysterese
		    DB 0,0     ;  ( h : min ) 2. Einschaltzeit
		    DB 0,0     ;  ( d : m   ) mit Datum
		    DB 0,0     ;  ( h : min ) 2. Ausschaltzeit
		    DB 0,0     ;  ( d : m   ) mit Datum
		    DB 0,0     ;  ( min : s ) Kurzzeitwecker
       .org %06FE
;------------------------------------------------------------------------------
; Sprungverteiler + Daten fr PR1 bis PR9 ( 1. Ebene )
;------------------------------------------------------------------------------
        .org %0700
TST:    JP     PRP     ; Sprung nach Dekadenprogrammierung
        .org %0703     ; ( PR1 )
        .byte  %FE     ; --> Daten fr Stelle 3    (R6)
        .byte  %40     ; --> Daten fr Zeigerreg.  (R8)
        .byte  %04     ; --> Daten fr LED-Anzeige (R9)
        .org %0706     ; ( PR4 )
        .byte  %F2     ; --> Daten fr Stelle 3    (R6)
        .byte  %48     ; --> Daten fr Zeigerreg.  (R8)
        .byte  %00     ; --> Daten fr LED-Anzeige (R9)
        .org %0709     ; ( PR7 )
        .byte  %FE     ; --> Daten fr Stelle 3    (R6)
        .byte  %4E     ; --> Daten fr Zeigerreg.  (R8)
        .byte  %08     ; --> Daten fr LED-Anzeige (R9)
        
        .org %070C
MA1:    JP     PR0     ; --> Sprung nach Anzeige Uhr;Temp.;Datum
        .org %070F     ; ( PR2 )
        .byte  %FE     ; --> Daten fr Stelle 3    (R6)
        .byte  %42     ; --> Daten fr Zeigerreg.  (R8)
        .byte  %02     ; --> Daten fr LED-Anzeige (R9)
        .org %0712     ; ( PR5 )
        .byte  %E2     ; --> Daten fr Stelle 3    (R6)
        .byte  %54     ; --> Daten fr Zeigerreg.  (R8)
        .byte  %00     ; --> Daten fr LED-Anzeige (R9)
        .org %0715     ; ( PR8 )
        .byte  %FE     ; --> Daten fr Stelle 3    (R6)
        .byte  %56     ; --> Daten fr Zeigerreg.  (R8)
        .byte  %08     ; --> Daten fr LED-Anzeige (R9)

        .org %0718
MA2:    JP     PRM     ; --> Sprung zur Weckeraktivierung       
        .org %071B     ; ( PR3 )
        .byte  %FD     ; --> Daten fr Stelle 3    (R6)
        .byte  %27     ; --> Daten fr Zeigerreg.  (R8)
        .byte  %00     ; --> Daten fr LED-Anzeige (R9)
        .org %071E     ; ( PR4 )
        .byte  %FE     ; --> Daten fr Stelle 3    (R6)
        .byte  %4A     ; --> Daten fr Zeigerreg.  (R8)
        .byte  %08     ; --> Daten fr LED-Anzeige (R9)
        .org %0721     ; ( PR9 )
        .byte  %FE     ; --> Daten fr Stelle 3    (R6)
        .byte  %5A     ; --> Daten fr Zeigerreg.  (R8)
        .byte  %08     ; --> Daten fr LED-Anzeige (R9)
;------------------------------------------------------------------------------
; Sprungverteiler + Daten fr PR11 bis PR19 ( 2. Ebene )
;------------------------------------------------------------------------------
        .org %0724
ZWE:    JP     PRP     ; Sprung nach Dekadenprogrammierung
        .org %0727     ; ( PR11 )
        .byte  %FC     ; --> Daten fr Stelle 3    (R6)
        .byte  %44     ; --> Daten fr Zeigerreg.  (R8)
        .byte  %04     ; --> Daten fr LED-Anzeige (R9)
        .org %072A     ; ( PR41 )
        .byte  %FE     ; --> Daten fr Stelle 3    (R6)
        .byte  %5E     ; --> Daten fr Zeigerreg.  (R8)
        .byte  %00     ; --> Daten fr LED-Anzeige (R9)
        .org %072D     ; ( PR71 )
        .byte  %FD     ; --> Daten fr Stelle 3    (R6)
        .byte  %50     ; --> Daten fr Zeigerreg.  (R8)
        .byte  %08     ; --> Daten fr LED-Anzeige (R9)
        
        .org %0730
MA4:    JP     PR0     ; --> Sprung nach Anzeige Uhr;Temp.;Datum
        .org %0733     ; ( PR21 )
        .byte  %FC     ; --> Daten fr Stelle 3    (R6)
        .byte  %46     ; --> Daten fr Zeigerreg.  (R8)
        .byte  %02     ; --> Daten fr LED-Anzeige (R9)
        .org %0736     ; ( PR51 )
        .byte  %FF     ; --> Daten fr Stelle 3    (R6)
        .byte  %08     ; --> Daten fr Zeigerreg.  (R8)
        .byte  %00     ; --> Daten fr LED-Anzeige (R9)
        .org %0739     ; ( PR81 )
        .byte  %FD     ; --> Daten fr Stelle 3    (R6)
        .byte  %58     ; --> Daten fr Zeigerreg.  (R8)
        .byte  %08     ; --> Daten fr LED-Anzeige (R9)

        .org %073C
MA3:    JP     PRM     ; --> Sprung zur Weckeraktivierung       
        .org %073F     ; ( PR31 )
        .byte  %2E     ; --> Daten fr Stelle 3    (R6)
        .byte  %52     ; --> Daten fr Zeigerreg.  (R8)
        .byte  %00     ; --> Daten fr LED-Anzeige (R9)
        .org %0742     ; ( PR41 )
        .byte  %FD     ; --> Daten fr Stelle 3    (R6)
        .byte  %4C     ; --> Daten fr Zeigerreg.  (R8)
        .byte  %08     ; --> Daten fr LED-Anzeige (R9)
        .org %0745     ; ( PR91 )
        .byte  %FD     ; --> Daten fr Stelle 3    (R6)
        .byte  %5C     ; --> Daten fr Zeigerreg.  (R8)
        .byte  %08     ; --> Daten fr LED-Anzeige (R9)
;------------------------------------------------------------------------------
; Sprungverteiler + Dekadenprogrammierung ( 3. Ebene )
;------------------------------------------------------------------------------
        .org %0748
        
ZAH:    JP     PRP       ; Sprung fr Dekadenprogrammierung
        LD     R10,#%01  ; gedrckte Zahl fr Dekadenprog. bergeben
        RET
        LD     R10,#%04  ; gedrckte Zahl fr Dekadenprog. bergeben
        RET
        LD     R10,#%07  ; gedrckte Zahl fr Dekadenprog. bergeben
        RET
        LD     R10,#%00  ; gedrckte Zahl fr Dekadenprog. bergeben
        RET
        LD     R10,#%02  ; gedrckte Zahl fr Dekadenprog. bergeben
        RET
        LD     R10,#%05  ; gedrckte Zahl fr Dekadenprog. bergeben
        RET
        LD     R10,#%08  ; gedrckte Zahl fr Dekadenprog. bergeben
        RET
        LD     R10,#%0F  ; gedrckte Zahl fr Dekadenprog. bergeben
        RET
        LD     R10,#%03  ; gedrckte Zahl fr Dekadenprog. bergeben
        RET
        LD     R10,#%06  ; gedrckte Zahl fr Dekadenprog. bergeben
        RET
        LD     R10,#%09  ; gedrckte Zahl fr Dekadenprog. bergeben
        RET
;------------------------------------------------------------------------------
; Anfang der Weckmelodie 1
;------------------------------------------------------------------------------
        .org %076C
M1A:    ; Teilerwerte fr T1 ( in Hexa )
        DB %B6,%B6,%A1,%A1,%90,%90,%90,%90,%DD,%DD,%DD,%DD,%B6,%B6,%B6,%02
        DB %B6,%B6,%F7,%F7,%DD,%DD,%B6,%B6,%B6,%B6,%F7,%F7,%DD,%DD,%DD,%DD
        DB %B6,%B6,%A1,%A1,%90,%90,%90,%90,%DD,%DD,%DD,%02,%DD,%DD,%DD,%DD
        DB %F7,%F7,%DD,%DD,%B6,%B6,%A1,%A1 ; 56 Bytes

;   Teilerwert f. T1           Vorteiler        = 3        = 5
;      in hex    in BCD        f(SOLL) f(IST)  f. 5MHz    bzw. 8MHz
;        B6       182            1100           1145       1099
;        A1       161            1246           1294       1242
;        90       144            1393           1447       1389
;        DD       221             905            943        905 
;        F7       247             806            843        810
;        02       002         > 20000         104167     100000
;
        
        .org %07A3

;------------------------------------------------------------------------------
; Anfang der Weckmelodie 1
;------------------------------------------------------------------------------
        .org %07A4
M2A:    DB %72,%72,%7A,%7A,%72,%72,%7A,%7A,%72,%72,%7A,%7A,%72,%72,%64,%64
        DB %72,%72,%72,%02,%7A,%7A,%7A,%7A,%7A,%7A,72,%72,%7A,%7A,%7A,%7A
        DB %8A,%8A,%8A,%02,%8A,%8A,%8A,%8A,%8A,%8A,%7A,%7A,%8A,%8A,%8A,%8A
        DB %99,%99,%99,%99,%02,%99,%99,%99,%99,%99,%99,%99 ; ( 64 Bytes )
        
;   Teilerwert f. T1           Vorteiler        = 3        = 5
;      in hex    in BCD        f(SOLL) f(IST)  f. 5MHz    bzw. 8MHz
;        72       114            2200           1827       2193
;        7A       122            2053           1708       2049
;        64       100            2493           2083       2500
;        8A       138            1812           1510       1812 
;        99       153            1634           1362       1634
;        02       002         > 20000         104167     100000
;
        
;        .org %07E3
;------------------------------------------------------------------------------
; Tabelle fr Monatsendzahl
;------------------------------------------------------------------------------
       .org    %07E4
       .byte %FF         ; mgl. Zugriff bei nicht geladenen Monat(00)
MON:   .byte %31         ; Januar
       .byte %28         ; Februar
       .byte %31         ; Mrz
       .byte %30         ; April
       .byte %31         ; Mai
       .byte %30         ; Juni
       .byte %31         ; Juli
       .byte %31         ; August
       .byte %30         ; September
       .byte %31         ; Oktober
       .byte %30         ; November
       .byte %31         ; Dezember
       .org  %07F1
       .byte %FF
;------------------------------------------------------------------------------
; Interruptvektorroutine fr Betriebssystem
;------------------------------------------------------------------------------
        .org    %07F2
IVR:    PUSH    RP
        CALL    TIM          ; --> Timerprogramm f. smtliche Zhlregister
        CALL    ANZ          ; --> Anzeigeprogramm ( ST1-ST5 + LED )
        CALL    TAS          ; --> Abfrage der Zusatztaste ( P 07 )
        POP     RP
        IRET
   
   end